import pyfie
import matplotlib.pyplot as plt

# 利便のため PyFIE 関数が返すエラーコードに応じて例外を発生させる機能を有効化
pyfie.ctrl.enable_f_err_exception(True)

# 処理対象画像の読み込み
image = pyfie.imread("houghcircle_input.png")

# ソーベルフィルタを利用したエッジ検出
edges = pyfie.F_DEDGE.PTR()  # 取得したエッジ
nedges = pyfie.INT()  # 取得したエッジ数
pyfie.fnFIE_edge_sobel_subpix(
    image, None, pyfie.F_EDGE_SOBEL_PARAMS(mag_threshold=120, nms_length=3),
    pyfie.F_EDGE_FEAT_DIRECT, pyfie.F_BORDER_NONE, (0.0, 0.0),
    edges, nedges
)

# 画像オブジェクトのプロット
image.imshow()
# エッジ点をプロット
edges.plot(nedges, c="lime", s=0.5)
# プロットした画像の表示
pyfie.plot_show()

# エッジ点を円ハフ検出用に変換
dedges = pyfie.DEDGE_T.ARRAY(nedges)
dedges.value = [pyfie.DEDGE_T(e.x, e.y, e.q) for e in edges[:nedges]]

# データ受け取り用変数を用意
circles = pyfie.HCD_CIRCLE_T.PTR()  # 出力円の配列
ncircles = pyfie.INT()  # 検出された円の数量

# 円の検出(接頭辞を省略したPyFIE関数使用)
pyfie.hough_circles_detect(
    dedges, len(dedges),  # 入力点列
    dRMin=20,  # 検出目標にする円の最小半径
    dRMax=250,  # 検出目標にする円の最大半径
    dRsl=1.0,  # 分解能
    iQRange=10,  # エッジ方向の片幅(度単位)
    iEdgePercent=30,  # 円上のエッジ点数と円周の百分率の閾値
    iMinScore=8,  # 円上のエッジ点数の閾値
    iCircleColor=pyfie.BLACK_WHITE_COLOR,  # 円の色
    bRefine=True,  # ロバスト推定法による検出円のパラメータを算出
    iMergeXY=2,  # 検出された円の統括する円心範囲サイズ
    iMergeR=1,  # 検出された円の統括する円半径範囲サイズ
    vppCircles=circles,  # 出力円の配列
    ipCirclesNum=ncircles,  # 検出された円の数量
    ipPntsIndex=None,  # 検出された円上の点のindex
)

# 取得した円を表示
print("num circles:", ncircles)
for i, circle in enumerate(circles[:ncircles]):
    print("#{}: xc = {:.2f}, yc = {:.2f}, r = {:.2f}, score = {}".format(
        i, circle.xc, circle.yc, circle.r, circle.score
    ))

# 画像オブジェクトのプロット
image.imshow()
# 取得した直線をプロット
ax = plt.gca()
for c in circles[:ncircles]:
    ax.add_artist(plt.Circle((c.xc, c.yc), c.r, color="red", fill=False))
# 結果をプロットした画像の保存
plt.savefig("hough_circle_result.png")
# 結果をプロットした画像の表示
pyfie.plot_show()

# 解放処理
edges.fnOAL_free()
circles.fnOAL_free()
